import 'dart:io';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:flutter_svg_provider/flutter_svg_provider.dart' as svg_provider;
import 'package:path/path.dart';

class ImageUtils {
  static ImageProvider getAssetImage(String name, {AssetBundle? bundle, String? package}) {
    return AssetImage(getImagePath(name), bundle: bundle, package: package);
  }

  static String getImagePath(String name) {
    if(name.startsWith("assets")) {
      return name;
    }
    return 'assets/images/$name'; 
  }

  static ImageProvider getImageProvider(String imageUrl, {String holderImg = 'assets/common/images/img_fail.png'}) {
    if (imageUrl.isEmpty) {
      return AssetImage(getImagePath(holderImg));
    } else {
      String ext = extension(File(imageUrl).path).toLowerCase();
      if (imageUrl.startsWith('http')) {
        if(ext == ".svg") {
          return svg_provider.Svg(imageUrl, source: svg_provider.SvgSource.network);
        }
        return CachedNetworkImageProvider(imageUrl);
      } else {
        return FileImage(File(imageUrl));
      }
    }
  }

  static Widget getNetworkImage(String imageUrl, {String holderImg = 'assets/common/images/img_fail.png', BoxFit? fit,}) {
    if (imageUrl.isEmpty) {
      return Image.asset(getImagePath(holderImg), fit: fit,);
    }
    String ext = (extension(File(imageUrl).path)).toLowerCase();
    if(ext == ".svg") {
      return SvgPicture.network(imageUrl);
    }
    return CachedNetworkImage(
        fit: fit,
        imageUrl: imageUrl,
        progressIndicatorBuilder: (context, url, downloadProgress) => 
                SizedBox(width: 30, height: 30, child: Center(child: CircularProgressIndicator(value: downloadProgress.progress))),
        errorWidget: (context, url, error) => const Icon(Icons.error),
     );
  }
}