import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tengits_common/utils/src/loading_toast_util.dart';
import 'package:tengits_common/widgets/src/button.dart';
import 'package:tengits_common/widgets/src/card.dart';
import 'package:tengits_widgets/calendar/table_calendar.dart';

import 'common_bottom_sheet.dart';

class DateRangeBottomSheet extends StatefulWidget {
  const DateRangeBottomSheet({Key? key, required this.title, this.initialStartTime, this.initialEndTime, this.selectMode}) : super(key: key);

  final String title;
  final DateTime? initialStartTime;
  final DateTime? initialEndTime;
  final RangeSelectionMode? selectMode;

  @override
  State<DateRangeBottomSheet> createState() => _DateRangeBottomSheetState();
}

class _DateRangeBottomSheetState extends State<DateRangeBottomSheet> {
  DateTime? startTime;
  DateTime? endTime;

  @override
  void initState() {
    super.initState();
    startTime = widget.initialStartTime;
    endTime = widget.initialEndTime;
  }

  @override
  Widget build(BuildContext context) {
    return CommonBottomSheet(
      title: widget.title,
      contentWidget: Container(
        constraints: BoxConstraints(maxHeight: 400.h),
        child: SingleChildScrollView(
          physics: const AlwaysScrollableScrollPhysics(),
          child: TableCalendar(
            locale: "zh",
            firstDay: DateTime(2000, 1, 1),
            lastDay: DateTime(2050, 12, 31),
            focusedDay: startTime ?? DateTime.now(),
            rangeStartDay: startTime,
            rangeEndDay: endTime,
            calendarFormat: CalendarFormat.month,
            rangeSelectionMode: widget.selectMode ?? RangeSelectionMode.toggledOff,
            onDaySelected: (selectedDay, focusedDay) {
              setState(() {
                startTime = selectedDay;
              });
            },
            onRangeSelected: (start, end, focusedDay) {
              setState(() {
                startTime = start;
                endTime = end;
              });
            },
          ),
          // child: TBrnCalendarView(
          //   key: isReset ? key1 : key2,
          //   displayMode: DisplayMode.month,
          //   selectMode: widget.selectMode ?? SelectMode.range,
          //   initStartSelectedDate: startTime,
          //   initEndSelectedDate: endTime,
          //   initDisplayDate: startTime ?? DateTime.now(),
          //   dateChange: (DateTime datetime) {
          //     setState(() {
          //       startTime = datetime;
          //       endTime = datetime;
          //       isReset = false;
          //     });
          //   },
          //   rangeDateChange: (range) {
          //     setState(() {
          //       startTime = range.start;
          //       endTime = range.end;
          //       isReset = false;
          //     });
          //   },
          // ),
        ),
      ),
      operatorWidget: ButtonOperatorCard(buttons: [
        OutBorderButton(
            onPress: () {
              setState(() {
                startTime = widget.initialStartTime;
                endTime = widget.initialEndTime;
              });
            },
            title: "重置"),
        PrimaryButton(
            onPress: () {
              if (widget.selectMode == RangeSelectionMode.toggledOff) {
                if (startTime == null) {
                  TLoadingUtils.showToast("请先选择时间");
                  return;
                }
                Navigator.of(context).pop(DateTimeRange(start: startTime!, end: startTime!));
              } else if (widget.selectMode == RangeSelectionMode.toggledOn) {
                if (startTime == null || endTime == null) {
                  TLoadingUtils.showToast("请先选择时间范围");
                  return;
                }
                Navigator.of(context).pop(DateTimeRange(start: startTime!, end: endTime!));
              }
            },
            title: "确定")
      ]),
    );
  }
}
