import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tengits_common/utils/src/image_utils.dart';

class TLCardStatisticsModel<T> {
  String? title;
  TextStyle? titleTextStyle;
  String? titleUnit;
  String? imagePath;
  List<T> items;

  TLCardStatisticsModel({
    this.title,
    this.titleTextStyle,
    this.titleUnit,
    this.items = const [],
  });
}

// 卡片统计
class TLCardstatistics<T> extends StatefulWidget {
  const TLCardstatistics({
    Key? key,
    required this.cardstatisticsModel,
    this.itemTitleTransformer,
    this.itemAmountTransformer,
    this.itemUnitTransformer,
    this.child,
    this.fontFamily = "dinFont",
  }) : super(key: key);

  final TLCardStatisticsModel cardstatisticsModel;
  final String Function(T item)? itemTitleTransformer;
  final String Function(T item)? itemAmountTransformer;
  final String Function(T item)? itemUnitTransformer;
  final Widget? child;
  final String fontFamily;

  @override
  State<TLCardstatistics<T>> createState() => _TLCardstatisticsState<T>();
}

class _TLCardstatisticsState<T> extends State<TLCardstatistics<T>> {
  ///
  late TLCardStatisticsModel _cardstatisticsModel;

  ///
  late ScrollController? _scrollController;

  late Timer? _timer;

  bool _maxScrollLocation = false;

  ScrollController? get defaultScrollController {
    return _cardstatisticsModel.items.length > 4 ? ScrollController() : null;
  }

  _scrollEvent() {
    if (_maxScrollLocation) {
      // _scrollController?.jumpTo(_scrollController?.position.minScrollExtent ?? 0);
      _scrollController?.animateTo(_scrollController?.position.minScrollExtent ?? 0, duration: const Duration(microseconds: 2000), curve: Curves.linear);
      _maxScrollLocation = false;
    } else {
      // _scrollController?.jumpTo(_scrollController?.position.maxScrollExtent ?? 0);
      _scrollController?.animateTo(_scrollController?.position.maxScrollExtent ?? 0, duration: const Duration(microseconds: 2000), curve: Curves.linear);
      _maxScrollLocation = true;
    }
  }

  @override
  void initState() {
    ///
    _cardstatisticsModel = widget.cardstatisticsModel;
    _scrollController = defaultScrollController;

    _timer = Timer.periodic(const Duration(seconds: 5), (timer) async {
      _scrollEvent();
    });

    super.initState();
  }

  @override
  void didUpdateWidget(TLCardstatistics<T> oldWidget) {
    ///
    if (oldWidget.cardstatisticsModel != widget.cardstatisticsModel) {
      ///
      _cardstatisticsModel = widget.cardstatisticsModel;
      _scrollController = defaultScrollController;
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    ///
    _scrollController?.dispose();

    ///
    _timer?.cancel();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Stack(children: [
      Container(
        height: 200.h,
        width: double.infinity,
        margin: EdgeInsets.only(left: 8.w, top: 20.h, right: 8.w, bottom: 16.h),
        padding: EdgeInsets.only(left: 8.w, top: 24.h, right: 8.w, bottom: 8.h),
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(8.r),
          image: DecorationImage(
            image: ImageUtils.getAssetImage("assets/images/total_bg.png", package: "tengits_common"),
            fit: BoxFit.fill,
          ),
        ),
        alignment: Alignment.topLeft,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Padding(
              padding: EdgeInsets.symmetric(horizontal: 8.h),
              child: Text(
                _cardstatisticsModel.title ?? "",
                style: _cardstatisticsModel.titleTextStyle ??
                    TextStyle(
                      fontFamily: widget.fontFamily,
                      color: Colors.white,
                      fontSize: 54.sp,
                      fontWeight: FontWeight.w600,
                    ),
                maxLines: 1,
              ),
            ),
            Padding(
              padding: EdgeInsets.symmetric(
                horizontal: 8.h,
              ),
              child: Text(
                _cardstatisticsModel.titleUnit ?? "",
                style: TextStyle(
                  color: Colors.white,
                  fontSize: 13.sp,
                  height: 1.5.h,
                ),
              ),
            ),
          ],
        ),
      ),
      Positioned(
        right: 8.w,
        top: -3.h,
        child: Image(
          image: ImageUtils.getAssetImage(_cardstatisticsModel.imagePath ?? ""),
          fit: BoxFit.fill,
        ),
      ),
      widget.child ??
          Positioned(
            bottom: 24.h,
            left: 16.w,
            right: 16.w,
            height: 76.h,
            child: Container(
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(8.r),
                color: Colors.white.withOpacity(0.2),
              ),
              child: LayoutBuilder(
                builder: (BuildContext context, BoxConstraints constraints) {
                  // 父视图最大宽度值
                  var viewWidth = constraints.maxWidth;
                  // 约定固定显示4个，并计算子项宽度，多的滚动查看
                  int itemNumber = _cardstatisticsModel.items.length > 4 ? 4 : _cardstatisticsModel.items.length;
                  double itemExtent = itemNumber > 1 ? (viewWidth - (itemNumber - 1) * 1.w) / itemNumber : 0;
                  return ListView(
                    controller: _scrollController,
                    physics: _cardstatisticsModel.items.length > 4 ? const AlwaysScrollableScrollPhysics() : const NeverScrollableScrollPhysics(),
                    padding: EdgeInsets.zero,
                    scrollDirection: Axis.horizontal,
                    shrinkWrap: true,
                    itemExtent: itemExtent,
                    children: _cardstatisticsModel.items
                        .asMap()
                        .keys
                        .map(
                          (index) => Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            mainAxisSize: MainAxisSize.min,
                            crossAxisAlignment: CrossAxisAlignment.center,
                            children: [
                              Container(
                                width: 0.5.w,
                                height: 24.h,
                                color: index == 0 ? Colors.transparent : Colors.white,
                              ),
                              Column(
                                mainAxisAlignment: MainAxisAlignment.center,
                                mainAxisSize: MainAxisSize.min,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  Text(
                                    widget.itemTitleTransformer?.call(_cardstatisticsModel.items[index] as T) ?? "",
                                    style: TextStyle(
                                      color: Colors.white,
                                      fontSize: 13.sp,
                                      height: 1.5.h,
                                    ),
                                  ),
                                  RichText(
                                    text: TextSpan(
                                        text: widget.itemAmountTransformer?.call(_cardstatisticsModel.items[index] as T) ?? "",
                                        style: TextStyle(
                                          fontFamily: widget.fontFamily,
                                          color: Colors.white,
                                          fontSize: 24.sp,
                                        ),
                                        children: [
                                          TextSpan(
                                            text: widget.itemUnitTransformer?.call(_cardstatisticsModel.items[index] as T) ?? "",
                                            style: TextStyle(
                                              color: Colors.white,
                                              fontSize: 8.sp,
                                            ),
                                          )
                                        ]),
                                  ),
                                ],
                              ),
                              Container(
                                width: 0.5.w,
                                height: 24.h,
                                color: index == _cardstatisticsModel.items.length - 1 ? Colors.transparent : Colors.white,
                              ),
                            ],
                          ),
                        )
                        .toList(),
                  );
                },
              ),
            ),
          ),
    ]);
  }
}
